package com.zhentao.userRelationships.util;


import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;

import java.util.regex.Pattern;

/**
 * 拼音处理工具类，用于获取汉字的拼音首字母和全拼
 */
public class PinyinUtil {

    // 配置拼音输出格式
    private static HanyuPinyinOutputFormat format = new HanyuPinyinOutputFormat();

    static {
        format.setCaseType(HanyuPinyinCaseType.UPPERCASE);    // 大写
        format.setToneType(HanyuPinyinToneType.WITHOUT_TONE); // 无音调
        format.setVCharType(HanyuPinyinVCharType.WITH_V);     // 用v表示ü
    }

    /**
     * 获取字符串的首字母拼音缩写
     * @param str 输入字符串，支持中文、英文、数字、符号
     * @return 首字母缩写（大写），非字母字符返回"#"
     */
    public static String getFirstLetter(String str) {
        if (str == null || str.trim().isEmpty()) {
            return "#";
        }

        char firstChar = str.trim().charAt(0);

        // 处理中文
        if (isChinese(firstChar)) {
            try {
                String pinyin = PinyinHelper.toHanyuPinyinStringArray(firstChar, format)[0];
                return String.valueOf(pinyin.charAt(0));
            } catch (BadHanyuPinyinOutputFormatCombination e) {
                return "#";
            }
        }
        // 处理英文字母
        else if (Character.isLetter(firstChar)) {
            return String.valueOf(Character.toUpperCase(firstChar));
        }
        // 处理数字、符号等
        else {
            return "#";
        }
    }

    /**
     * 获取字符串的完整拼音
     * @param str 输入字符串
     * @return 拼音字符串（首字母大写，其余小写，空格分隔）
     */
    public static String getFullPinyin(String str) {
        if (str == null || str.trim().isEmpty()) {
            return "";
        }

        StringBuilder pinyin = new StringBuilder();
        for (char c : str.toCharArray()) {
            if (isChinese(c)) {
                try {
                    String[] pinyinArray = PinyinHelper.toHanyuPinyinStringArray(c, format);
                    if (pinyinArray != null && pinyinArray.length > 0) {
                        pinyin.append(pinyinArray[0]);
                    } else {
                        pinyin.append(c);
                    }
                } catch (BadHanyuPinyinOutputFormatCombination e) {
                    pinyin.append(c);
                }
            } else {
                pinyin.append(c);
            }
            pinyin.append(" ");
        }

        // 首字母大写，其余小写
        if (pinyin.length() > 0) {
            String result = pinyin.toString().trim();
            return result.substring(0, 1).toUpperCase() + result.substring(1).toLowerCase();
        }

        return "";
    }

    /**
     * 判断字符是否为中文
     */
    private static boolean isChinese(char c) {
        return c >= 0x4E00 && c <= 0x9FA5;
    }

    /**
     * 判断字符串是否包含中文
     */
    public static boolean containsChinese(String str) {
        if (str == null || str.trim().isEmpty()) {
            return false;
        }

        Pattern pattern = Pattern.compile("[\\u4E00-\\u9FA5]");
        return pattern.matcher(str).find();
    }
}
